/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is the reusable ccl library
 * (http://www.kclee.com/clemens/java/ccl/).
 *
 * The Initial Developer of the Original Code is Chr. Clemens Lee.
 * Portions created by Chr. Clemens Lee are Copyright (C) 1999
 * Chr. Clemens Lee. All Rights Reserved.
 *
 * Contributor(s): Chr. Clemens Lee <clemens@kclee.com>
 */

package ccl.util;

import java.util.Vector;

/**
 * Used by the Init class. It represents an application
 * Option and it's value.
 *
 * @version $Id: Option.java,v 1.5 2000/04/05 16:57:30 clemens Exp clemens $
 * @author <a href="http://www.kclee.com/clemens/">
 *         Chr. Clemens Lee</a>
 *         &lt;<a href="mailto:clemens@kclee.com">
 *         clemens@kclee.com
 *         </a>>
 */
public class Option {
    public static final int BOOLEAN = 0;
    public static final int STRING = 1;
    public static final int INTEGER = 2;

    private String[] _asType = {"b", "s"};

    private String _sName;
    private String _sDescription;
    private int _type;
    private Object _oValue;
    private boolean _bMandatory = false;

    public Option( String sName_, String sInitString_ ) 
    {
        _sName = sName_;
	Util.debug( this, "<init>(..).sInitString_: " + sInitString_ );
        Vector vProperties = Util.stringToLines(sInitString_, ',');
        Util.debug("Option.Option(String): vProperties: " + vProperties);
        // Up to now no other types are allowed.
        if (vProperties.elementAt(0).equals(_asType[BOOLEAN])) {
            _type = BOOLEAN;
        } else if (vProperties.elementAt(0).equals(_asType[STRING])) {
            _type = STRING;
        } else {
            Util.panicIf(true, "Option.Option(String): Type not allowed: " + vProperties.elementAt(0));
        }
        if (((String)vProperties.elementAt(1)).charAt(0) == 'm') {
            _bMandatory = true;
        }
        _sDescription = (String)vProperties.elementAt(2);
        Util.debug("Option.Option(String): _sName: " + _sName);
        Util.debug("Option.Option(String): _sDescription: " + _sDescription);
        Util.debug("Option.Option(String): _bMandatory: " + _bMandatory);
        Util.debug("Option.Option(String): _type: " + _type);
    }

    public String getName() {
        return _sName;
    }

    public String getDescription() {
        return _sDescription;
    }

    public boolean isMandatory() {
        return _bMandatory;
    }

    public int getType() {
        return _type;
    }
}
